import { CreateUserBody, LoginUserBody, UserInfo } from '@core';
import { Services } from '../app';
import { UserWithPassword } from '../entities/user';

export function LoginUser(
  services: Services,
): (tracker: string, raw: LoginUserBody) => Promise<UserInfo | null> {
  const { userModel } = services;

  return async (tracker, raw) => {
    const user = await userModel.login(tracker, raw);
    return user ? user.Info() : null;
  };
}

export function CreateUser(
  services: Services,
): (tracker: string, raw: CreateUserBody) => Promise<UserInfo | null> {
  const { userModel } = services;

  return async (tracker, raw) => {
    const user = await userModel.create(tracker, new UserWithPassword(raw));
    return user ? user.Info() : null;
  };
}

export function ReadUser(
  services: Services,
): (tracker: string, uuid: string) => Promise<UserInfo | null> {
  const { userModel } = services;

  return async (tracker, uuid) => {
    const user = await userModel.read(tracker, uuid);
    return user ? user.Info() : null;
  };
}
